﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace WG_CheckBoxHeaderWithGroupByRows
{
    class CustomCreationFilter : IUIElementCreationFilter
    {
        public EventHandler<GroupByRowCheckBoxClickedEventArgs> GroupByRowCheckBoxClicked;
        private string _columnKey;
        private System.Windows.Forms.CheckState _state = System.Windows.Forms.CheckState.Indeterminate;

        public System.Windows.Forms.CheckState State
        {
            get { return _state; }
            set { _state = value; }
        }

        public CustomCreationFilter(string columnKey)
        {
            this._columnKey = columnKey;
        }

        public void AfterCreateChildElements(UIElement parent)
        {
            if (parent is GroupByRowUIElement)
            {
                var rowUIElement = parent as GroupByRowUIElement;
                var groupDescElement = parent.GetDescendant(typeof(GroupByRowDescriptionUIElement));
                CheckBoxUIElement checkBoxEl = null;
                //check if the row contains any rows and if it has the desired column
                if (!(rowUIElement.GroupByRow.Rows.Count > 0 && rowUIElement.GroupByRow.Rows[0].HasCell("Check")))
                    return;

                if (rowUIElement.GroupByRow.Tag != null)
                {
                    checkBoxEl = (CheckBoxUIElement)rowUIElement.GroupByRow.Tag;
                    //substitute the temp checkbox with proper one when the GroupByRowUIElement is first created
                    if (!(checkBoxEl.Parent is GroupByRowUIElement &&
                        ((GroupByRowUIElement)checkBoxEl.Parent).GroupByRow == rowUIElement.GroupByRow))
                    {
                        checkBoxEl = CreateCheckBox(rowUIElement, checkBoxEl.CheckState);
                        rowUIElement.GroupByRow.Tag = checkBoxEl;
                    }
                }
                else
                {
                    checkBoxEl = CreateCheckBox(rowUIElement, System.Windows.Forms.CheckState.Unchecked);
                    rowUIElement.GroupByRow.Tag = checkBoxEl;
                }


                if (_state != System.Windows.Forms.CheckState.Indeterminate)
                {
                    checkBoxEl.CheckState = _state;
                }
                

                checkBoxEl.Rect = new Rectangle(groupDescElement.Rect.Location, 
                    new Size(groupDescElement.Rect.Height, groupDescElement.Rect.Height));
                groupDescElement.Rect = new Rectangle(checkBoxEl.Rect.Right, checkBoxEl.Rect.Top,
                    groupDescElement.Rect.Width - checkBoxEl.Rect.Width, groupDescElement.Rect.Height);
                parent.ChildElements.Add(checkBoxEl);

            }
        }

        private CheckBoxUIElement CreateCheckBox(GroupByRowUIElement rowUIElement, System.Windows.Forms.CheckState checkState)
        {
            CheckBoxUIElement checkBoxEl;
            checkBoxEl = new CheckBoxUIElement(rowUIElement);
            checkBoxEl.CheckState = checkState;
            rowUIElement.GroupByRow.Tag = checkBoxEl;
            checkBoxEl.ElementClick += checkBoxEl_ElementClick;
            return checkBoxEl;
        }


        void checkBoxEl_ElementClick(object sender, UIElementEventArgs e)
        {
            //don't handle the events inside the filter as it may cause memory leaks
            if (GroupByRowCheckBoxClicked != null)
            {
                GroupByRowCheckBoxClicked(sender, new GroupByRowCheckBoxClickedEventArgs(e.Element));
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }
    }


}
