﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace WG_CheckBoxHeaderWithGroupByRows
{
    public partial class Form1 : Form
    {
        CustomCreationFilter filter;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SetUpGridDataSource();
            ultraGrid1.Rows.ExpandAll(true);
            filter = new CustomCreationFilter("Check");
            ultraGrid1.CreationFilter = filter;
            filter.GroupByRowCheckBoxClicked += filter_GroupByRowCheckBoxClicked;
        }

        private void filter_GroupByRowCheckBoxClicked(object sender, GroupByRowCheckBoxClickedEventArgs e)
        {
            if(e.GroupByRow.Rows.Count > 0)
            {
                var cells = e.GroupByRow.Rows.Select(r => r.Cells["Check"]).ToList();
                
                bool isChecked = e.CheckBox.CheckState == CheckState.Checked ? true : false;
                //added check
                if (e.CheckBox.CheckState != CheckState.Indeterminate)
                {
                    foreach (var cell in cells)
                    {
                        cell.Value = isChecked;
                    }
                }

                CheckState state = cells[0].Column.GetHeaderCheckedState(e.GroupByRow.Rows);
                filter.State = state;
            }
        }


        private void SetUpGridDataSource()
        {
            DataSet ds = new DataSet();
            DataTable t1 = ds.Tables.Add("table1");
            t1.Columns.Add("id", typeof(int)).AutoIncrement = true;
            t1.Columns.Add("Name", typeof(string));
            t1.Columns.Add("Quantity", typeof(int));
            t1.Columns.Add("Price", typeof(double));
            t1.Columns.Add("Date", typeof(DateTime));

            ds.Tables["table1"].Rows.Add(new Object[] { null, "Beverage", 50, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Appliances", 5, 90, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "FastFood", 20, 32, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Furniture", 12, 79, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Meat", 40, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Fruits", 25, 12, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Vegetables", 30, 13, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Cars", 13, 200, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Tools", 60, 15.3, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Estate", 5, 3000, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Motorcycles", 10, 320, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Clothes", 40, 69, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Shoes", 24, 40.6, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Games", 53, 50, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Books", 42, 22, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new Object[] { null, "Movies", 38, 19, new DateTime(2009, 02, 28) });
            
            

            ultraGrid1.DataSource = ds;
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            var c = e.Layout.Bands[0].Columns.Add("Check");
            c.DataType = typeof(bool);
            c.Header.Caption = "";
            c.Header.CheckBoxAlignment = HeaderCheckBoxAlignment.Center ;
            c.Header.CheckBoxVisibility = HeaderCheckBoxVisibility.Always;
            c.Header.CheckBoxSynchronization = HeaderCheckBoxSynchronization.Band;
            
            e.Layout.Bands[0].SortedColumns.Add(c.Band.Columns["Date"], false, true);

        }

        private void ultraGrid1_CellChange(object sender, CellEventArgs e)
        {
            if (e.Cell.Column.Key == "Check" && e.Cell.Row.ParentRow.IsGroupByRow)
            {
                UpdateGroupByRowCheckBox((UltraGridGroupByRow)e.Cell.Row.ParentRow);               
            }
        }

        private void UpdateGroupByRowCheckBox(UltraGridGroupByRow parentRow)
        {
            ultraGrid1.ActiveRow.Update();
            var checkBox = parentRow.Tag as CheckBoxUIElement;
            if (checkBox != null)
            {
                if ((parentRow.Rows.Count > 0 && parentRow.Rows[0].HasCell("Check")))
                {
                    var values = parentRow.Rows.Select(r => r.GetCellValue("Check")).ToList();

                    if (values.All(v => v.Equals(true)))
                    {
                        checkBox.CheckState = CheckState.Checked;
                    }
                    else if (values.All(v => v.Equals(false)))
                    {
                        checkBox.CheckState = CheckState.Unchecked;
                    }
                    else
                    {
                        checkBox.CheckState = CheckState.Indeterminate;
                    }
                }
            }
        }

        private void ultraGrid1_AfterHeaderCheckStateChanged(object sender, AfterHeaderCheckStateChangedEventArgs e)
        {
            CheckState state = e.Column.GetHeaderCheckedState(e.Rows);
            filter.State = state;
            ultraGrid1.Refresh();
        }

        private void ultraGrid1_AfterRowExpanded(object sender, RowEventArgs e)
        {
            if (e.Row.IsGroupByRow )
            {
                var rows = ((UltraGridGroupByRow)e.Row).Rows;
                if (rows.IsGroupByRows)
                {
                    foreach (var row in rows)
                    {
                        if (row.Tag == null)
                        {
                            //temp UIElement just to store the actual CheckedState
                            row.Tag = new CheckBoxUIElement(ultraGrid1.DisplayLayout.UIElement);
                        }
                        UpdateGroupByRowCheckBox((UltraGridGroupByRow)row);
                    }
                    
                }
            }
        }

        private void ultraGrid1_AfterSortChange(object sender, BandEventArgs e)
        {
            
            var rows = e.Band.GetRowEnumerator(GridRowType.GroupByRow).Cast<UltraGridGroupByRow>();
            if(rows != null && rows.Count() > 0)
            {
                foreach (var row in rows)
                {
                    if (row.Tag == null)
                    {
                        //temp UIElement just to store the actual CheckedState
                        row.Tag = new CheckBoxUIElement(ultraGrid1.DisplayLayout.UIElement);
                        UpdateGroupByRowCheckBox(row);
                    }
                }
            }
        }
   

    }
}
